﻿using System;
using System.Collections;
using System.Data;
using System.Data.Sql;
using System.Data.SqlClient;
using System.Drawing;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;
using System.Collections.Generic;

namespace CreateExcelReport
{
    class ExcelUILanguageHelper : IDisposable
    {
        private CultureInfo m_CurrentCulture;

        public ExcelUILanguageHelper()
        {
            // save current culture and set culture to en-US
            m_CurrentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
        }

        // return to normal culture
        public void Dispose() => Thread.CurrentThread.CurrentCulture = m_CurrentCulture;
    }

    class Program
    {
        private const string sqlselectManagers =
            @"select 
                employees.Name, employees.JobTitle, 
                employees.OrganizationLevel,
                employees.Department,
                managers.name [Reports to], 
                case when employees.JobTitle != managers.JobTitle then managers.JobTitle else '' end [Manager Job Title]
            from 
                (
                    SELECT
                        e.[BusinessEntityID], 
                        p.FirstName + case when p.MiddleName is not null then ' ' + p.MiddleName else '' end + ' ' + p.LastName [Name],
                        e.[JobTitle], d.[Name] AS [Department], edh.[StartDate], 
                        case when e.OrganizationLevel is null then 0 else e.OrganizationLevel end OrganizationLevel,
                        case when e.OrganizationNode is null then '/' else e.OrganizationNode.GetAncestor(1).ToString() end OrganizationNode
                    FROM Person.Person p
                        join HumanResources.Employee e on e.BusinessEntityID = p.BusinessEntityID
                        JOIN [HumanResources].[EmployeeDepartmentHistory] edh ON e.[BusinessEntityID] = edh.[BusinessEntityID] 
                        JOIN [HumanResources].[Department] d ON edh.[DepartmentID] = d.[DepartmentID] 
                    WHERE edh.EndDate IS NULL
                ) employees
                join
                (
                    SELECT
                        e.[BusinessEntityID], 
                        p.FirstName + case when p.MiddleName is not null then ' ' + p.MiddleName else '' end + ' ' + p.LastName [Name],
                        e.[JobTitle], d.[Name] AS [Department], edh.[StartDate], 
                        case when e.OrganizationLevel is null then 0 else e.OrganizationLevel end OrganizationLevel,
                        case when e.OrganizationNode is null then '/' else e.OrganizationNode.ToString() end OrganizationNode
                    FROM Person.Person p
                        join HumanResources.Employee e on e.BusinessEntityID = p.BusinessEntityID
                        JOIN [HumanResources].[EmployeeDepartmentHistory] edh ON e.[BusinessEntityID] = edh.[BusinessEntityID] 
                        JOIN [HumanResources].[Department] d ON edh.[DepartmentID] = d.[DepartmentID] 
                    WHERE edh.EndDate IS NULL
                ) managers
                on employees.OrganizationNode = managers.OrganizationNode
            group by employees.department, employees.JobTitle, employees.OrganizationLevel,
                managers.name, 
                managers.JobTitle, 
                employees.[name], employees.BusinessEntityID, employees.StartDate
            order by employees.OrganizationLevel,  employees.JobTitle";

        private const string connectionString = "Server=TCP:192.168.0.7;Database=AdventureWorks2019;Integrated Security=SSPI;Connect Timeout=180;";

        static void Main(string[] args)
        {
            Excel.Application xlsApp;
            Excel.Workbook xlsWorkbook;
            Excel.Worksheet xlsSmartArt;
            Excel.Worksheet xlsManagers;
            object misValue = System.Reflection.Missing.Value;

            // Save employees for levels 1 and 2
            List<List<string>> topExec = new List<List<string>>();
            string ceo = string.Empty;

            using(new ExcelUILanguageHelper())
            {
                string fileName = Environment.GetFolderPath(Environment.SpecialFolder.Desktop) + "\\" + "ExcelReport-SmartArt.xlsx";

                // Remove the old excel report file
                try
                {
                    FileInfo oldFile = new FileInfo(fileName);
                    if(oldFile.Exists)
                    {
                        File.SetAttributes(oldFile.FullName, FileAttributes.Normal);
                        oldFile.Delete();
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error removing old Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }

                try
                {
                    // In case that database cannot be open, don't open Excel app
                    using SqlConnection conn = new SqlConnection(connectionString);
                    conn.Open();

                    xlsApp = new Excel.Application();
                    xlsWorkbook = xlsApp.Workbooks.Add(misValue);
                    xlsWorkbook.Sheets.Add(); // Add a second sheet

                    // Create the header for the first sheet
                    xlsManagers = (Excel.Worksheet)xlsWorkbook.Sheets[1];
                    xlsManagers.Name = "Company Organization Structure";
                    xlsManagers.Cells[1, 1] = "Company Organization Structure";
                    Excel.Range rangeManagers = xlsManagers.get_Range("A1", "F1");
                    rangeManagers.Merge(1);
                    rangeManagers.Borders.Color = Color.Black.ToArgb();
                    rangeManagers.Interior.Color = Color.Yellow.ToArgb();
                    rangeManagers.Font.Name = "Courier New";
                    rangeManagers.Font.Size = 18;
                    rangeManagers.Cells.HorizontalAlignment = Excel.XlHAlign.xlHAlignCenterAcrossSelection;

                    // Process managers
                    int i = 3;

                    using SqlCommand cmdmanagers = new SqlCommand(sqlselectManagers, conn);
                    using SqlDataReader drmanager = cmdmanagers.ExecuteReader();
                    if(drmanager.HasRows)
                    {
                        for(int j = 0; j < drmanager.FieldCount; ++j)
                        {
                            xlsManagers.Cells[i, j + 1] = drmanager.GetName(j);
                        }
                        ++i;
                    }

                    while(drmanager.Read())
                    {
                        for(int j = 1; j <= drmanager.FieldCount; ++j)
                        {
                            xlsManagers.Cells[i, j] = drmanager.GetValue(j - 1);
                        }

                        // Check for CEO and level 1 and 2 in the company
                        int level = drmanager.GetInt32(2);
                        if(level == 0) // CEO
                            ceo = drmanager.GetString(0);
                        else if(level == 1 || level == 2)
                        {
                            topExec.Add(new List<string>()
                            {
                                drmanager.GetString(0),
                                drmanager.GetString(1),
                                level.ToString(),
                                drmanager.GetString(4)
                            });
                        }

                        ++i;
                    }
                    drmanager.Close();

                    rangeManagers = xlsManagers.get_Range("A2", "I" + (i + 2).ToString());
                    rangeManagers.Columns.AutoFit();

                    // Create the SmartArt Horizontal Hierarchy
                    xlsSmartArt = (Excel.Worksheet)xlsWorkbook.Sheets[2];
                    xlsSmartArt.Name = "Company Organization Chart";
                    xlsManagers.Cells[1, 1] = "Company Organization Chart";
                    Excel.Range rangeChart = xlsManagers.get_Range("A1", "H1");
                    rangeChart.Merge(1);
                    rangeChart.Borders.Color = Color.Black.ToArgb();
                    rangeChart.Interior.Color = Color.Yellow.ToArgb();
                    rangeChart.Font.Name = "Courier New";
                    rangeChart.Font.Size = 14;
                    rangeChart.Cells.HorizontalAlignment = Excel.XlHAlign.xlHAlignCenterAcrossSelection;

                    Office.SmartArtLayout layout = xlsApp.SmartArtLayouts[106]; // SmartArt Horizontal Hierarchy index
                    Excel.Shape shape = xlsSmartArt.Shapes.AddSmartArt(layout, 50, 50, 700, 2000);
                    shape.AlternativeText = "Company Org Chart";

                    if(shape.HasSmartArt == Office.MsoTriState.msoTrue)
                    {
                        Office.SmartArt smartArt = shape.SmartArt;
                        Office.SmartArtNodes allNodes = smartArt.AllNodes;
                        Office.SmartArtNode ceoNode = null;

                        // Remove the nodes added by the template
                        foreach(Office.SmartArtNode node in allNodes)
                        {
                            if(node.Level != 1)
                            {
                                node.Delete();
                            }
                            else
                            {
                                ceoNode = node;
                                ceoNode.TextFrame2.TextRange.Text = ceo + "\n" + "Chief Executive Officer";
                            }
                        }

                        // For each person in level 1 create a node
                        List<Office.SmartArtNode> level1 = new List<Office.SmartArtNode>();
                        foreach(List<string> list in topExec)
                        {
                            if(list[2] == "1")
                            {
                                Office.SmartArtNode node = ceoNode.AddNode(Office.MsoSmartArtNodePosition.msoSmartArtNodeBelow);
                                node.TextFrame2.TextRange.Text = list[0] + "\n" + list[1];
                                level1.Add(node);
                            }
                        }

                        // Add level 2 people
                        foreach(List<string> list in topExec)
                        {
                            if(list[2] == "2")
                            {
                                foreach(Office.SmartArtNode node in level1)
                                {
                                    string text = node.TextFrame2.TextRange.Text;
                                    text = text.Substring(0, text.IndexOf('\n'));
                                    if(text == list[3])
                                    {
                                        Office.SmartArtNode node2 = node.AddNode(Office.MsoSmartArtNodePosition.msoSmartArtNodeBelow);
                                        node2.TextFrame2.TextRange.Text = list[0] + "\n" + list[1];
                                    }
                                }
                            }
                        }
                    }

                    // Save the Excel file
                    xlsWorkbook.SaveAs(fileName, Excel.XlFileFormat.xlWorkbookDefault, misValue, misValue, misValue, misValue,
                        Excel.XlSaveAsAccessMode.xlExclusive, Excel.XlSaveConflictResolution.xlLocalSessionChanges, misValue, misValue, misValue, misValue);
                    xlsWorkbook.Close(true, misValue, misValue);
                    xlsApp.Quit();

                    ReleaseObject(xlsWorkbook);
                    ReleaseObject(xlsApp);

                    if(MessageBox.Show("Excel report has been created on your desktop\nWould you like to open it?", "Created Excel report",
                        MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                    {
                        Process.Start(fileName);
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error creating Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
            }
        }

        static private void ReleaseObject(object obj)
        {
            try
            {
                System.Runtime.InteropServices.Marshal.ReleaseComObject(obj);
                obj = null;
            }
            catch(Exception ex)
            {
                obj = null;
                MessageBox.Show("Exception occured while releasing object " + ex.ToString());
            }
            finally
            {
                GC.Collect();
            }
        }
    }
}
